@@ -28,6 +28,8 @@ expect_r6_class <- function(object, class) {
   expect_s3_class(object, "R6")
 }
 
+#' Mask `testthat::expect_equal()` in order to compare ArrowObjects using their
+#' `Equals` methods from the C++ library.
 expect_equal <- function(object, expected, ignore_attr = FALSE, ..., info = NULL, label = NULL) {
   if (inherits(object, "ArrowObject") && inherits(expected, "ArrowObject")) {
     mc <- match.call()
@@ -64,24 +66,35 @@ verify_output <- function(...) {
   testthat::verify_output(...)
 }
 
-#' @param expr A dplyr pipeline with `input` as its start
-#' @param tbl A tbl/df as reference, will make RB/Table with
-#' @param skip_record_batch string skip message, if should skip RB test
-#' @param skip_table string skip message, if should skip Table test
-#' @param warning string expected warning from the RecordBatch and Table paths,
-#'   passed to `expect_warning()`. Special values:
+#' Ensure that dplyr methods on Arrow objects return the same as for data frames
+#'
+#' This function compares the output of running a dplyr expression on a tibble
+#' or data.frame object against the output of the same expression run on
+#' Arrow Table and RecordBatch objects.
+#'
+#'
+#' @param expr A dplyr pipeline which must have `.input` as its start
+#' @param tbl A tibble or data.frame which will be substituted for `.input`
+#' @param skip_record_batch The skip message to show (if you should skip the
+#' RecordBatch test)
+#' @param skip_table The skip message to show (if you should skip the Table test)
+#' @param warning The expected warning from the RecordBatch and Table comparison
+#'  paths, passed to `expect_warning()`. Special values:
 #'     * `NA` (the default) for ensuring no warning message
 #'     * `TRUE` is a special case to mean to check for the
 #'      "not supported in Arrow; pulling data into R" message.
 #' @param ... additional arguments, passed to `expect_equal()`
-expect_dplyr_equal <- function(expr,
-                               tbl,
-                               skip_record_batch = NULL,
-                               skip_table = NULL,
-                               warning = NA,
-                               ...) {
+compare_dplyr_binding <- function(expr,
+                                  tbl,
+                                  skip_record_batch = NULL,
+                                  skip_table = NULL,
+                                  warning = NA,
+                                  ...) {
+
+  # Quote the contents of `expr` so that we can evaluate it a few different ways
   expr <- rlang::enquo(expr)
-  expected <- rlang::eval_tidy(expr, rlang::new_data_mask(rlang::env(input = tbl)))
+  # Get the expected output by evaluating expr on the .input data.frame using regular dplyr
+  expected <- rlang::eval_tidy(expr, rlang::new_data_mask(rlang::env(.input = tbl)))
 
   if (isTRUE(warning)) {
     # Special-case the simple warning:
@@ -91,11 +104,12 @@ expect_dplyr_equal <- function(expr,
 
   skip_msg <- NULL
 
+  # Evaluate `expr` on a RecordBatch object and compare with `expected`
   if (is.null(skip_record_batch)) {
     expect_warning(
       via_batch <- rlang::eval_tidy(
         expr,
-        rlang::new_data_mask(rlang::env(input = record_batch(tbl)))
+        rlang::new_data_mask(rlang::env(.input = record_batch(tbl)))
       ),
       warning
     )
@@ -104,11 +118,12 @@ expect_dplyr_equal <- function(expr,
     skip_msg <- c(skip_msg, skip_record_batch)
   }
 
+  # Evaluate `expr` on a Table object and compare with `expected`
   if (is.null(skip_table)) {
     expect_warning(
       via_table <- rlang::eval_tidy(
         expr,
-        rlang::new_data_mask(rlang::env(input = arrow_table(tbl)))
+        rlang::new_data_mask(rlang::env(.input = arrow_table(tbl)))
       ),
       warning
     )
@@ -122,21 +137,35 @@ expect_dplyr_equal <- function(expr,
   }
 }
 
-expect_dplyr_error <- function(expr, # A dplyr pipeline with `input` as its start
-                               tbl, # A tbl/df as reference, will make RB/Table with
-                               ...) {
+#' Assert that Arrow dplyr methods error in the same way as methods on data.frame
+#'
+#' Comparing the error message generated when running expressions on R objects
+#' against the error message generated by running the same expression on Arrow
+#' Tables and RecordBatches.
+#'
+#' @param expr A dplyr pipeline which must have `.input` as its start
+#' @param tbl A tibble or data.frame which will be substituted for `.input`
+#' @param ... additional arguments, passed to `expect_error()`
+compare_dplyr_error <- function(expr,
+                                tbl,
+                                ...) {
   # ensure we have supplied tbl
   force(tbl)
 
   expr <- rlang::enquo(expr)
   msg <- tryCatch(
-    rlang::eval_tidy(expr, rlang::new_data_mask(rlang::env(input = tbl))),
+    rlang::eval_tidy(expr, rlang::new_data_mask(rlang::env(.input = tbl))),
     error = function(e) {
       msg <- conditionMessage(e)
 
+      # For compatibility with dplyr 1.0.8
+      if (grepl("Problem while computing", msg[1])) {
+        msg <- conditionMessage(e$parent)
+      }
+
       # The error here is of the form:
       #
-      # Problem with `filter()` input `..1`.
+      # Problem with `filter()` .input `..1`.
       # x object 'b_var' not found
       # ℹ Input `..1` is `chr == b_var`.
       #
@@ -158,7 +187,7 @@ expect_dplyr_error <- function(expr, # A dplyr pipeline with `input` as its star
   expect_error(
     rlang::eval_tidy(
       expr,
-      rlang::new_data_mask(rlang::env(input = record_batch(tbl)))
+      rlang::new_data_mask(rlang::env(.input = record_batch(tbl)))
     ),
     msg,
     ...
@@ -166,27 +195,36 @@ expect_dplyr_error <- function(expr, # A dplyr pipeline with `input` as its star
   expect_error(
     rlang::eval_tidy(
       expr,
-      rlang::new_data_mask(rlang::env(input = arrow_table(tbl)))
+      rlang::new_data_mask(rlang::env(.input = arrow_table(tbl)))
     ),
     msg,
     ...
   )
 }
 
-expect_vector_equal <- function(expr, # A vectorized R expression containing `input` as its input
-                                vec, # A vector as reference, will make Array/ChunkedArray with
-                                skip_array = NULL, # Msg, if should skip Array test
-                                skip_chunked_array = NULL, # Msg, if should skip ChunkedArray test
-                                ignore_attr = FALSE, # ignore attributes?
-                                ...) {
+#' Comparing the output of running expressions on R vectors against the same
+#' expression run on Arrow Arrays and ChunkedArrays.
+#'
+#' @param expr A vectorized R expression which must have `.input` as its start
+#' @param vec A vector which will be substituted for `.input`
+#' @param skip_array The skip message to show (if you should skip the Array test)
+#' @param skip_chunked_array The skip message to show (if you should skip the ChunkedArray test)
+#' @param ignore_attr Ignore differences in specified attributes?
+#' @param ... additional arguments, passed to `expect_as_vector()`
+compare_expression <- function(expr,
+                               vec,
+                               skip_array = NULL,
+                               skip_chunked_array = NULL,
+                               ignore_attr = FALSE,
+                               ...) {
   expr <- rlang::enquo(expr)
-  expected <- rlang::eval_tidy(expr, rlang::new_data_mask(rlang::env(input = vec)))
+  expected <- rlang::eval_tidy(expr, rlang::new_data_mask(rlang::env(.input = vec)))
   skip_msg <- NULL
 
   if (is.null(skip_array)) {
     via_array <- rlang::eval_tidy(
       expr,
-      rlang::new_data_mask(rlang::env(input = Array$create(vec)))
+      rlang::new_data_mask(rlang::env(.input = Array$create(vec)))
     )
     expect_as_vector(via_array, expected, ignore_attr, ...)
   } else {
@@ -199,7 +237,7 @@ expect_vector_equal <- function(expr, # A vectorized R expression containing `in
 
     via_chunked <- rlang::eval_tidy(
       expr,
-      rlang::new_data_mask(rlang::env(input = ChunkedArray$create(split_vector[[1]], split_vector[[2]])))
+      rlang::new_data_mask(rlang::env(.input = ChunkedArray$create(split_vector[[1]], split_vector[[2]])))
     )
     expect_as_vector(via_chunked, expected, ignore_attr, ...)
   } else {
@@ -211,15 +249,24 @@ expect_vector_equal <- function(expr, # A vectorized R expression containing `in
   }
 }
 
-expect_vector_error <- function(expr, # A vectorized R expression containing `input` as its input
-                                vec, # A vector as reference, will make Array/ChunkedArray with
-                                skip_array = NULL, # Msg, if should skip Array test
-                                skip_chunked_array = NULL, # Msg, if should skip ChunkedArray test
-                                ...) {
+#' Comparing the error message generated when running expressions on R objects
+#' against the error message generated by running the same expression on Arrow
+#' Arrays and ChunkedArrays.
+#'
+#' @param expr An R expression which must have `.input` as its start
+#' @param vec A vector which will be substituted for `.input`
+#' @param skip_array The skip message to show (if you should skip the Array test)
+#' @param skip_chunked_array The skip message to show (if you should skip the ChunkedArray test)
+#' @param ... additional arguments, passed to `expect_error()`
+compare_expression_error <- function(expr,
+                                     vec,
+                                     skip_array = NULL,
+                                     skip_chunked_array = NULL,
+                                     ...) {
   expr <- rlang::enquo(expr)
 
   msg <- tryCatch(
-    rlang::eval_tidy(expr, rlang::new_data_mask(rlang::env(input = vec))),
+    rlang::eval_tidy(expr, rlang::new_data_mask(rlang::env(.input = vec))),
     error = function(e) {
       msg <- conditionMessage(e)
 
@@ -240,7 +287,7 @@ expect_vector_error <- function(expr, # A vectorized R expression containing `in
     expect_error(
       rlang::eval_tidy(
         expr,
-        rlang::new_data_mask(rlang::env(input = Array$create(vec)))
+        rlang::new_data_mask(rlang::env(.input = Array$create(vec)))
       ),
       msg,
       ...
@@ -256,7 +303,7 @@ expect_vector_error <- function(expr, # A vectorized R expression containing `in
     expect_error(
       rlang::eval_tidy(
         expr,
-        rlang::new_data_mask(rlang::env(input = ChunkedArray$create(split_vector[[1]], split_vector[[2]])))
+        rlang::new_data_mask(rlang::env(.input = ChunkedArray$create(split_vector[[1]], split_vector[[2]])))
       ),
       msg,
       ...
